#!/usr/bin/perl -T
########################################################################
#                                                                      #
#  Cookie Test Routine                           Revised: 2003-04-05   #
#                                                                      #
#  Bob Swift                                                           #
#  R.S. Digital Solutions                                              #
#  http://www.rsds.ca                                                  #
#  software@rsds.ca                                                    #
#                                                                      #
#  Copyright (c) 2003, R.S. Digital Solutions.  All rights reserved.   #
#                                                                      #
#  This program is free software; you can redistribute it and/or       #
#  modify it under the terms of the GNU General Public License as      #
#  published by the Free Software Foundation.                          #
#                                                                      #
#  This program is distributed in the hope that it will be useful,     #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of      #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       #
#  GNU General Public License for more details.                        #
#                                                                      #
#  You should have received a copy of the GNU General Public           #
#  License along with this program; if not, write to:                  #
#                                                                      #
#      The Free Software Foundation, Inc.                              #
#      59 Temple Place, Suite 330                                      #
#      Boston, MA  02111-1307                                          #
#      United States of America                                        #
#                                                                      #
########################################################################
#                                                                      #
#  This script can be called from a form or directly from from a       #
#  web page through an HREF= setting in an anchor block, such as:      #
#                                                                      #
#       <A HREF="/cgi-bin/cookie_test.cgi">Cookie Test</A>             #
#                                                                      #
#  Fields passed to the routine during the call are:                   #
#                                                                      #
#       success={URL Identifier}      # Optional, Default = 0 #        #
#       failure={URL Identifier}      # Optional, Default = 0 #        #
#                                                                      #
#  The URL Identifiers specified for the success and failure           #
#  parameters, indicate which URL to display in the event of a         #
#  successful (or unsuccessful) test.  The actual URLs are coded       #
#  below in the configuration section.  If the success or failure      #
#  parameters are omitted, then the values will default to 0.          #
#                                                                      #
#  Multiple success and failure URLs can be specified within the       #
#  configuration section of the script to accommodate allowing         #
#  different areas of the site to handle the results of the test       #
#  in different ways, without having to install multiple copies        #
#  of the cookie test script.                                          #
#                                                                      #
#  When the script is called, it will attempt to send a cookie to      #
#  the remote user.  It then sets an http redirect to call itself      #
#  with the action "testresult".  Any "success" and "failure"          #
#  parameters passed with the initial call are preserved.              #
#                                                                      #
#  The "testresult" action reads the cookie information provided       #
#  by the remote user's browser, and checks for the value set          #
#  during the initial call to the script.  If found, the specified     #
#  "success" URL is displayed, otherwise the specified "failure"       #
#  URL is loaded.                                                      #
#                                                                      #
#  The script performs some rudamentary security checks upon           #
#  start-up, and will redirect to a specified security breach URL      #
#  if the checks fail.                                                 #
#                                                                      #
########################################################################

########################################################################
#                                                                      #
#  Configuration section.                                              #
#                                                                      #
########################################################################

# Path and name of this CGI file
$cgifile = '/cgi-bin/cookie_test.cgi';

# URL to display on test success (entered as 'ID', 'URL' pairs)
%success = (
             '0', '/cookie_okay.html',
	     '1', '/pool/login.html',
	     '2', '/shopping/login.html'
	   );

# URL to display on test failure (entered as 'ID', 'URL' pairs)
%failure = (
             '0', '/cookie_fail.html',
	     '1', '/pool/need_cookie.html',
	     '2', '/shopping/need_cookie.html'
	   );

# URL to display on security violation
$security = '/cgi-bin/security_log.cgi';


########################################################################
#                                                                      #
#   End of configuration section.                                      #
#                                                                      #
########################################################################

#----------------------------------------------------------------------#

# Basic system security checks.

unless ($cgifile eq $ENV{"SCRIPT_NAME"} ) { security_error(); }
$temp = $ENV{"HTTP_REFERER"};
($temp) = ($temp =~ /^http:\/\/([^\/]+)/i);
unless ($temp eq $ENV{"HTTP_HOST"} ) { security_error(); }
$scriptcall = 'http://' . $temp . $cgifile;


#----------------------------------------------------------------------#

# Process script input arguments.

if ($ENV{'REQUEST_METHOD'} eq 'GET') {
  $saveinput = $ENV{'QUERY_STRING'};
  @pairs = split(/&/, $ENV{'QUERY_STRING'});
}
elsif ($ENV{'REQUEST_METHOD'} eq 'POST') {
  read (STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
  $saveinput = $buffer;
  @pairs = split(/&/, $buffer);
  if ($ENV{'QUERY_STRING'}) {
    @getpairs = split(/&/, $ENV{'QUERY_STRING'});
    push(@pairs, @getpairs);
  }
}

foreach $pair (@pairs) {
  ($key, $value) = split (/=/, $pair);
  $key =~ tr/+/ /;
  $key =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
  $key = lc($key);
  $value =~ tr/+/ /;
  $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
  $value =~s/<!--(.|\n)*-->//g;
  if ($formdata{$key}) { $formdata{$key} .= ", $value"; }
  else { $formdata{$key} = $value; }
}

unless (exists $formdata{"action"}) { $formdata{"action"} = '' }
unless (exists $formdata{"success"}) { $formdata{"success"} = '0' }
unless (exists $formdata{"failure"}) { $formdata{"failure"} = '0' }


#----------------------------------------------------------------------#

# Read the test cookie.

if ($formdata{"action"} =~ /^testresult$/i) {
  if (exists $success{$formdata{"success"}}) {
    $successurl = $success{$formdata{"success"}}; }
  else { $successurl = $success{$formdata{"0"}}; }

  if (exists $failure{$formdata{"failure"}}) {
    $failureurl = $failure{$formdata{"failure"}}; }
  else { $failureurl = $failure{$formdata{"0"}}; }

  my $a = get_cookie("COOKIETEST");
  if ($a =~ /^success$/i) { print_redirect($successurl); }
  else { print_redirect($failureurl); }
}


#----------------------------------------------------------------------#

# Write the test cookie.

else {
  my $temp = $ENV{"HTTP_HOST"};
  print "Set-cookie:COOKIETEST=success;path=$cgifile;domain=$temp\n";
  $scriptcall .= '?action=testresult&success=';
  $scriptcall .= $formdata{"success"};
  $scriptcall .= '&failure=';
  $scriptcall .= $formdata{"failure"};
  print_redirect($scriptcall);
}

exit 0;


########################################################################
#                       #                                              #
#   Subroutine          #   Returns the specified cookie value         #
#                       #                                              #
########################################################################
#                                                                      #
#   Input parameters required in the function call:                    #
#                                                                      #
#       cookie_key                                                     #
#                                                                      #
########################################################################

sub get_cookie {
  my $ret = '';
  my $a = shift;
  unless ( defined $a ) { $a = ''; }
  unless ($a eq '') {
    if ($ENV{"HTTP_COOKIE"}) {
      my @cookies = split(/;/, $ENV{"HTTP_COOKIE"});
      foreach my $cookie (@cookies) {
        (my $name1, my $value1) = split(/=/, $cookie);
	($name1) = ($name1 =~ /^\s*(.*)\s*$/i);
	($value1) = ($value1 =~ /^\s*(.*)\s*$/i);
        if ($name1 eq $a) { $ret = $value1; }
      }
    }
  }
  return $ret;
}


########################################################################
#                       #                                              #
#   Subroutine          #   Print url redirection directive            #
#                       #                                              #
########################################################################
#                                                                      #
#   Input parameter required in the function call:                     #
#                                                                      #
#       url                                                            #
#                                                                      #
########################################################################

sub print_redirect {
  print "Location: $_[0]\n\n";
  exit;
}


########################################################################
#                       #                                              #
#   Subroutine          #   Display the security error page            #
#                       #                                              #
########################################################################
#                                                                      #
#   There are no input parameters required in the function call.       #
#                                                                      #
########################################################################

sub security_error {
  #-------------------------------------------#
  #                                           #
  #  Insert any user-defined security breach  #
  #  logging or other actions at this point.  #
  #  These will be executed prior to the      #
  #  security error page being displayed.     #
  #                                           #
  #-------------------------------------------#
  print_redirect($security);
  exit;
}


########################################################################
#                                                                      #
#   End of script.                                                     #
#                                                                      #
########################################################################
